# -*- coding: utf-8 -*-

import curses.wrapper
import logging
import time
import math

from tuikit.container import Container
from tuikit.backend_curses import BackendCurses


class TopWindow(Container):
    
    '''Top window of an application. Covers entire screen.'''
    
    def __init__(self):
        '''Create top window.'''
        Container.__init__(self)

        self.focuswidget = None

        self.top = self

        self.timeout = []
        self.timelast = None

        self.connect('draw', self.on_draw)


    def keypress(self, keyname, char):
        if self.handle('keypress', keyname, char):
            return
        if self.focuswidget and self.focuswidget != self:
            self.focuswidget.emit('keypress', keyname, char)


    def on_draw(self, screen, x, y):
        screen.erase()


    def add_timeout(self, s, func):
        if not len(self.timeout):
            self.timelast = time.time()
        self.timeout += [[s, func]]


    def remove_timeout(self, func):
        for to in self.timeout[:]:
            if to[1] == func:
                self.timeout.remove(to)


    def has_timeout(self):
        return len(self.timeout) and True or False


    def nearest_timeout(self):
        return min(self.timeout)[0]


    def process_timeout(self):
        now = time.time()
        lasted = now - self.timelast
        self.timelast = now

        for to in self.timeout[:]:
            newt = to[0] - lasted
            if newt <= 0.0:
                to[1]()
                self.timeout.remove(to)
            else:
                to[0] = newt


class Application:
    
    '''Application class. Defines main loop.'''
       
    def __init__(self):
        '''Create application.'''
        
        '''Top window.'''
        self.top = TopWindow()
        self.quit = False
        
        '''Renderer class, i.e. BackendCurses.'''
        self.screen = None

        self.log = logging.getLogger('tuikit')
        self.log.setLevel(logging.DEBUG)
        handler = logging.FileHandler('./tuikit.log')
        formatter = logging.Formatter('%(asctime)s %(levelname)-5s %(message)s', '%y-%m-%d %H:%M:%S')
        handler.setFormatter(formatter)
        self.log.addHandler(handler)
        self.log.info('=== start ===')


    def start(self):
        '''Start application. Runs main loop.'''
        curses.wrapper(self.mainloop)


    def terminate(self):
        '''Terminate application.'''
        self.quit = True


    def mainloop(self, screen):
        '''The main loop.'''
        self.screen = BackendCurses(screen)
        self.applytheme()
        self.top.width, self.top.height = self.screen.width, self.screen.height
        self.top.emit('resize')

        while True:
            self.top.draw(self.screen)
            self.screen.commit()

            timeout = None
            if self.top.has_timeout():
                timeout = int(math.ceil(self.top.nearest_timeout() * 10))

            events = self.screen.process_input(timeout)

            if self.top.has_timeout():
                self.top.process_timeout()

            for event in events:
                if event[0] == 'resize':
                    self.top.width, self.top.height = self.screen.width, self.screen.height
                self.top.emit(event[0], *event[1:])

            if self.quit:
                break


    def applytheme(self):
        screen = self.screen
        screen.setcolor('normal',                  'white on black')
        screen.setcolor('strong',                  'white on black, bold')
        screen.setcolor('active',                  'black on cyan')
        screen.setcolor('window:normal',           'white on blue')
        screen.setcolor('window:controls',         'white on blue, bold')
        screen.setcolor('window:controls-active',  'cyan on blue, bold')
        screen.setcolor('button',                  'black on white')
        screen.setcolor('button-active',           'black on cyan')
        screen.setcolor('menu',                    'black on cyan')
        screen.setcolor('menu-active',             'white on cyan, bold')

